#include "../include/parse.h"
#include "../include/db.h"
#include <libpq-fe.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef MEMDEBUG
int num_malloc=0;

char * _malloc(int len)
{
    num_malloc++;
    return calloc( 1, len);
}

void _free(void *mem)
{
    num_malloc--;
    free(mem);
}
#else
char* _malloc(int len) { return calloc( 1, len); }
#define _free free
#endif

Plugin * ListOfPlugin=0;
Plugin * CurrentPlugin=0;
Plugin * ListOfRc=0;

char * PidFile = PIDFILE;

static void __addtolist(char *name_cfg, char *name_exec, char *pdir, char *cdir);
static void __addcommand(char *command, char *value, Command **Cmd);
static Host * __addhost(Plugin *p, char *key, char *psection, char *host);
static PutSection ** __getlast_ps(PutSection **ps);
static Host ** __getlast_h(Host **h);


int OpenRcFile()
{
PGconn *conn;
PGresult *res;
Plugin *p;
Host *h;
char * str[1024], *tmp;
int i,j;

	
    conn=connectDB();
    if(PQstatus(conn) == CONNECTION_BAD)
    {
#ifdef DEBUG
	printf("can`t establish connection to database '%s' as user '%s' \n(%s)\n\n",getenv("PG_DBNAME"), getenv("PG_USER"),PQerrorMessage(conn));
#endif
	PQfinish(conn);
	return OCF_CANNOTCONNECT;
    }
 	
//   Plugin

	p=ListOfRc=(Plugin *)_malloc(sizeof(Plugin));
	p->PathOfCfg=0;
	p->NameOfCfg=_malloc(sizeof("Rc lines")+1);
	strcpy(p->NameOfCfg,"Rc lines");
	p->PathOfExec=0;
	p->NameOfExec=0;
   
	
//  Plugin  Host
	
	sprintf (str,"select status,nameofexec,comment,color,loglevel from rc ORDER BY nameofexec,status;");
	res=PQexec(conn,str);
	if (PQresultStatus(res) != PGRES_TUPLES_OK) {
		PQclear(res);
		PQfinish(conn);
		return OCF_CANNOTREAD;
	}
	
	for (i=0;i<PQntuples(res);i++){
		h=__addhost(p,PQgetvalue(res,i,0),PQgetvalue(res,i,1),PQgetvalue(res,i,2));
		h->h_addcount=2;
		h->h_adddata=(char **) _malloc(2*sizeof(char *));
		if (PQgetisnull(res,i,3)){
			h->h_adddata[0]=NULL;
		}
		else {

			tmp=PQgetvalue(res,i,3);
			h->h_adddata[0]=(char *) _malloc(strlen(tmp)+1);
			strcpy(h->h_adddata[0],tmp);
		}

		if (PQgetisnull(res,i,4)){
			h->h_adddata[1]=NULL;
		}
		else {
			tmp=PQgetvalue(res,i,4);
#if DEBUG
	fprintf(stderr,"LogLevel of %d string is %s\n",i,tmp);
#endif
			h->h_adddata[1]=(char *) _malloc(strlen(tmp)+1);
			strcpy(h->h_adddata[1],tmp);
		}
	}
	
	PQclear(res);
	PQfinish(conn);

	return OCF_OK;


}

int OpenCfgFile(char *name_cfg, int flag)
{
    PGconn *conn;
    PGresult *res,*pres;
    int nFields, i, j, k, l, m;
    Plugin *p;
    Host *h;
    char str[256], *tmp;

    conn=connectDB();
    if(PQstatus(conn) == CONNECTION_BAD)
    {
#ifdef DEBUG
	printf("can`t establish connection to database '%s' as user '%s' \n(%s)\n\n",getenv("PG_DBNAME"), getenv("PG_USER"),PQerrorMessage(conn));
#endif
	PQfinish(conn);
	return OCF_CANNOTCONNECT;
    }
    
    if(flag & (FL_OPENMAINFILE|FL_OPENONLYMAINFILE))
    {

	// scan for all plugins and add to list
	res=PQexec(conn, "BEGIN");
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTBEGIN;
	}
	PQclear(res);
	
	res = PQexec(conn, "DECLARE mycursor CURSOR FOR select nameofcfg, nameofexec from eye");
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTDECLARECURSOR;
	}
	PQclear(res);

	res = PQexec(conn, "FETCH ALL in mycursor");
	if(PQresultStatus(res) != PGRES_TUPLES_OK)
	{
	    PQclear(res);
	}

	nFields = PQnfields(res);
	if(nFields != 2)
	{
	    PQfinish(conn);
	    return OCF_SELECTERROR;
	}
	
// retriving PluginDirectory
	pres=PQexec(conn, "select value from parametrs where nameofcfg='eyed' AND parametr='PluginDirectory';");

	for(i=0; i<PQntuples(res); i++)
	    __addtolist(PQgetvalue(res, i, 0), PQgetvalue(res, i, 1), PQgetvalue(pres,0,0), 0);
	
	PQclear(pres);
	PQclear(res);

	res = PQexec(conn, "CLOSE mycursor");
	PQclear(res);
	res = PQexec(conn, "END");
	PQclear(res);

    }else
	__addtolist(name_cfg, 0, 0, 0);

    // scan for all plugin parameters
    p=ListOfPlugin;
    while(p)
    {
	res=PQexec(conn, "BEGIN");
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTBEGIN;
	}
	PQclear(res);
	
	sprintf(str, "DECLARE mycursor CURSOR FOR select parametr, value from parametrs where nameofcfg='%s'", p->NameOfCfg);
	res = PQexec(conn, str);
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTDECLARECURSOR;
	}
	PQclear(res);

	res = PQexec(conn, "FETCH ALL in mycursor");
	if(PQresultStatus(res) != PGRES_TUPLES_OK)
	{
	    PQclear(res);
	}

	nFields = PQnfields(res);
	if(nFields != 2)
	{
	    PQfinish(conn);
	    return OCF_SELECTERROR;
	}
	for(i=0; i<PQntuples(res); i++)
	    __addcommand(PQgetvalue(res, i, 0), PQgetvalue(res, i, 1), &p->p_cmd);
	PQclear(res);

	res = PQexec(conn, "CLOSE mycursor");
	PQclear(res);
	res = PQexec(conn, "END");
	PQclear(res);

        p=p->p_next;
    } 

    if(flag & FL_OPENONLYMAINFILE) return OCF_OK;

    // scan for all plugin hosts info
    p=ListOfPlugin;
    while(p)
    {
	res=PQexec(conn, "BEGIN");
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTBEGIN;
	}
	PQclear(res);
	
	sprintf(str, "DECLARE mycursor CURSOR FOR select * from %s", p->NameOfCfg);
	res = PQexec(conn, str);
	if(PQresultStatus(res) != PGRES_COMMAND_OK)
	{
	    PQclear(res);
	    PQfinish(conn);
	    return OCF_CANNOTDECLARECURSOR;
	}
	PQclear(res);

	res = PQexec(conn, "FETCH ALL in mycursor");
	if(PQresultStatus(res) != PGRES_TUPLES_OK)
	{
	    PQclear(res);
	}

	nFields = PQnfields(res);
	k=l=m=-1;
	for(j=0; j<PQnfields(res); j++)
	{
	    tmp=PQfname(res, j);
	    if(!strcasecmp(tmp, "key")) k=j;
	    if(!strcasecmp(tmp, "putsection")) l=j;
	    if(!strcasecmp(tmp, "host")) m=j;
	}
	if(k==-1 || l==-1 || m==-1)
	{
	    PQfinish(conn);
	    return OCF_SELECTERROR;
	}
	for(i=0; i<PQntuples(res); i++)
	{
	    h=__addhost(p, PQgetvalue(res, i, k), PQgetvalue(res, i, l), PQgetvalue(res, i, m));
	    
	    j=PQnfields(res)-3;
	    if(j<=0) 
	    {
		h->h_adddata=0;
		continue;
	    }
	    
	    h->h_adddata=(char **)_malloc(j*sizeof(char*));
	    h->h_addcount=0;
	    for(j=0; j<PQnfields(res); j++)
		if(j!=k && j!=l && j!=m) 
		{
		    if(PQgetisnull(res,i,j)){
		    	h->h_adddata[h->h_addcount]=NULL;
		    }
		    else {
		    	tmp=PQgetvalue(res, i, j);
		    	h->h_adddata[h->h_addcount]=_malloc(strlen(tmp)+1);
		    	strcpy(h->h_adddata[h->h_addcount],tmp);
		    }
		    h->h_addcount++;
		}
	}
	PQclear(res);

	res = PQexec(conn, "CLOSE mycursor");
	PQclear(res);
	res = PQexec(conn, "END");
	PQclear(res);

        p=p->p_next;
    } 

    PQfinish(conn);
    return OCF_OK;
}

int CloseRcFile()
{
    Plugin *p1, *p2;
    PutSection *ps1, *ps2;
    Host *h1, *h2;
    int i=0;
	
    p1=ListOfRc;
    if(PidFile != PIDFILE) _free(PidFile);
    
	if(p1->NameOfCfg) _free(p1->NameOfCfg);
	if(p1->NameOfExec) _free(p1->NameOfExec);
	if(p1->PathOfCfg) _free(p1->PathOfCfg);
	if(p1->PathOfExec) _free(p1->PathOfExec);
    
	ps1=p1->ps_next;
	while(ps1)
	{
	    if(ps1->ps_name) _free(ps1->ps_name);
	    
	    h1=ps1->h_next;
	    while(h1)
	    {
		if(h1->_h_addr) _free(h1->_h_addr);
		
		for(i=0; i<h1->h_addcount; i++)
		    _free(h1->h_adddata[i]);
		
		h2=h1->h_next;
		_free(h1);	    
		h1=h2;
	    }
	
	    ps2=ps1->ps_next;
	    _free(ps1);
	    ps1=ps2;
	}
	
	_free(p1);

#ifdef MEMDEBUG
    printf("num of memory not freed - %i\n", num_malloc);
#endif

    return OCF_OK;
}

int CloseCfgFile()
{
    Plugin *p1, *p2;
    Command *c1, *c2;
    PutSection *ps1, *ps2;
    Host *h1, *h2;
    int i=0;
	
    p1=ListOfPlugin;
    if(PidFile != PIDFILE) _free(PidFile);
    
    while(p1)
    {
	if(p1->NameOfCfg) _free(p1->NameOfCfg);
	if(p1->NameOfExec) _free(p1->NameOfExec);
	if(p1->PathOfCfg) _free(p1->PathOfCfg);
	if(p1->PathOfExec) _free(p1->PathOfExec);
    
	c1=p1->p_cmd;
	while(c1)
	{
	    if(c1->c_cmd) _free(c1->c_cmd);
	    if(c1->c_value) _free(c1->c_value);
	    c2=c1->c_next;
	    _free(c1);
	    c1=c2;
	}
	
	ps1=p1->ps_next;
	while(ps1)
	{
	    if(ps1->ps_name) _free(ps1->ps_name);
	    
	    h1=ps1->h_next;
	    while(h1)
	    {
		if(h1->_h_addr) _free(h1->_h_addr);
		
		for(i=0; i<h1->h_addcount; i++)
		    _free(h1->h_adddata[i]);
		
		h2=h1->h_next;
		_free(h1);	    
		h1=h2;
	    }
	
	    ps2=ps1->ps_next;
	    _free(ps1);
	    ps1=ps2;
	}
	
	p2=p1->p_next;
	_free(p1);
	p1=p2;
    }
#ifdef MEMDEBUG
    printf("num of memory not freed - %i\n", num_malloc);
#endif

    return OCF_OK;
}

Plugin * GetFirstPlugin()
{
    if(!ListOfPlugin) return 0;
    CurrentPlugin = ListOfPlugin;
    return CurrentPlugin;
}

Plugin * GetFirstRc()
{
    if(!ListOfRc) return 0;
    return ListOfRc;
}

Plugin * GetNextPlugin()
{
    if(!CurrentPlugin) return 0;
    CurrentPlugin=CurrentPlugin->p_next;

    return CurrentPlugin;
}

/************************************************************************/
static void __addtolist(char *name_cfg, char *name_exec, char *pdir, char *cdir)
{
    Plugin **p=&ListOfPlugin;
    char *tmp;
    while(*p) p=&((*p)->p_next);
    
    *p=(Plugin *)_malloc(sizeof(Plugin));

    if(cdir)
    {
    	(*p)->PathOfCfg=_malloc(strlen(cdir)+1);
    	strcpy((*p)->PathOfCfg, cdir);
    }else
    	(*p)->PathOfCfg=0;
    (*p)->NameOfCfg=_malloc(strlen(name_cfg)+1);
    strcpy((*p)->NameOfCfg, name_cfg);

    if(pdir)
    {
        (*p)->PathOfExec=_malloc(strlen(pdir)+1);
        strcpy((*p)->PathOfExec, pdir);
    }
    else
        (*p)->PathOfExec=0;

    if(name_exec)
    {
        (*p)->NameOfExec=_malloc(strlen(name_exec)+1);
        strcpy((*p)->NameOfExec, name_exec);
    }else
        (*p)->NameOfExec=0;
}

static void __addcommand(char *command, char *value, Command **Cmd)
{
    Command **cmd=Cmd;

    while(*cmd) cmd=&((*cmd)->c_next);
    *cmd=(Command *)_malloc(sizeof(Command));
    (*cmd)->c_next=0;
    
    (*cmd)->c_cmd=_malloc(strlen(command)+1);
    strcpy((*cmd)->c_cmd, command);

    (*cmd)->c_value=_malloc(strlen(value)+1);
    strcpy((*cmd)->c_value, value);
}

static Host * __addhost(Plugin *p, char *key, char *psection, char *host)
{
    PutSection **ps;
    Host **h;
    int _key;
    _key=atoi(key);

    for(ps=&(p->ps_next); *ps; ps=&((*ps)->ps_next))
	if(!strcasecmp((*ps)->ps_name, psection)) break;

    if(*ps)
    {
	h=&((*ps)->h_next);
	while(*h) h=&((*h)->h_next);
    }else
    {
	*ps=(PutSection *)_malloc(sizeof(PutSection));
	(*ps)->ps_next=0;
	
	(*ps)->ps_name=_malloc(strlen(psection)+1);
	strcpy((*ps)->ps_name, psection);
	h=&((*ps)->h_next);
    }

    *h=(Host *)_malloc(sizeof(Host));
    (*h)->h_next=0;
    
    (*h)->h_key=_key;
    (*h)->_h_addr=_malloc(strlen(host)+1);
    strcpy((*h)->_h_addr, host);
    
    return *h;
}

PutSection ** __getlast_ps(PutSection **ps)
{
    PutSection **_ps=ps;
    while(*_ps) _ps=&((*_ps)->ps_next);
    return _ps;
}

Host ** __getlast_h(Host **h)
{
    Host **_h=h;
    while(*_h) _h=&((*_h)->h_next);
    return _h;
}
